@@ -9,7 +9,7 @@ import logging
 import unittest
 from unittest.mock import AsyncMock, MagicMock, call, patch
 
-from mcp_fuzzer.fuzzer.protocol_fuzzer import ProtocolFuzzer
+from mcp_fuzzer.fuzz_engine.fuzzer.protocol_fuzzer import ProtocolFuzzer
 
 
 class TestProtocolFuzzer(unittest.IsolatedAsyncioTestCase):
@@ -19,7 +19,8 @@ class TestProtocolFuzzer(unittest.IsolatedAsyncioTestCase):
         """Set up test fixtures."""
         # Create a mock transport for testing
         self.mock_transport = AsyncMock()
-        self.mock_transport.send_request.return_value = {"result": "test_response"}
+        # ProtocolFuzzer now uses send_raw to transmit envelope-level fuzzed messages
+        self.mock_transport.send_raw.return_value = {"result": "test_response"}
         self.fuzzer = ProtocolFuzzer(self.mock_transport)
 
     def test_init(self):
@@ -42,7 +43,7 @@ class TestProtocolFuzzer(unittest.IsolatedAsyncioTestCase):
         self.assertEqual(third_id, 3)
         self.assertEqual(self.fuzzer.request_id_counter, 3)
 
-    @patch("mcp_fuzzer.fuzzer.protocol_fuzzer.logging")
+    @patch("mcp_fuzzer.fuzz_engine.fuzzer.protocol_fuzzer.logging")
     async def test_fuzz_protocol_type_success(self, mock_logging):
         """Test successful fuzzing of a protocol type."""
         results = await self.fuzzer.fuzz_protocol_type("InitializeRequest", runs=3)
@@ -55,7 +56,7 @@ class TestProtocolFuzzer(unittest.IsolatedAsyncioTestCase):
             self.assertTrue("success" in result)
             self.assertEqual(result["run"], i + 1)
 
-    @patch("mcp_fuzzer.fuzzer.protocol_fuzzer.logging")
+    @patch("mcp_fuzzer.fuzz_engine.fuzzer.protocol_fuzzer.logging")
     async def test_fuzz_protocol_type_realistic_vs_aggressive(self, mock_logging):
         """Test that realistic and aggressive phases produce different results."""
         realistic_results = await self.fuzzer.fuzz_protocol_type(
@@ -83,11 +84,11 @@ class TestProtocolFuzzer(unittest.IsolatedAsyncioTestCase):
         # Should return empty list for unknown types
         self.assertEqual(len(results), 0)
 
-    @patch("mcp_fuzzer.fuzzer.protocol_fuzzer.logging")
+    @patch("mcp_fuzzer.fuzz_engine.fuzzer.protocol_fuzzer.logging")
     async def test_fuzz_protocol_type_transport_exception(self, mock_logging):
         """Test handling of transport exceptions."""
         # Set up transport to raise an exception
-        self.mock_transport.send_request.side_effect = Exception("Transport error")
+        self.mock_transport.send_raw.side_effect = Exception("Transport error")
 
         results = await self.fuzzer.fuzz_protocol_type("InitializeRequest", runs=2)
 
@@ -95,8 +96,10 @@ class TestProtocolFuzzer(unittest.IsolatedAsyncioTestCase):
         self.assertEqual(len(results), 2)
         for result in results:
             self.assertIn("server_error", result)
+        # Ensure send_raw was attempted for each run
+        self.assertEqual(self.mock_transport.send_raw.await_count, 2)
 
-    @patch("mcp_fuzzer.fuzzer.protocol_fuzzer.logging")
+    @patch("mcp_fuzzer.fuzz_engine.fuzzer.protocol_fuzzer.logging")
     async def test_fuzz_all_protocol_types(self, mock_logging):
         """Test fuzzing all protocol types."""
         results = await self.fuzzer.fuzz_all_protocol_types(runs_per_type=2)
